@@ -1,5 +1,5 @@
 /*
- $Id: lfe.c 1748 2015-07-10 20:29:00Z sgaure $
+ $Id: lfe.c 1788 2015-10-26 11:46:59Z sgaure $
 */
 #include "lfe.h"
 SEXP MY_threads(SEXP rt) {
@@ -20,7 +20,7 @@ static R_CallMethodDef callMethods[] = {
   {"dsyrk", (DL_FUNC) &MY_dsyrk, 4},
   {"dsyr2k", (DL_FUNC) &MY_dsyr2k, 5},
   {"address", (DL_FUNC) &MY_address, 1},
-  {"ppf", (DL_FUNC) &MY_ppf, 2},
+  //  {"ppf", (DL_FUNC) &MY_ppf, 2},
   //  {"threads", (DL_FUNC) &MY_threads, 1},
   {NULL, NULL, 0}
 };
